/*
 * Copyright  2008 Nokia Corporation.
 */

#include "CSndMixer.h"
#include "CMixerThread.h"
#include <e32svr.h>
#include "../engine/ApplicationPrefs.h"

_LIT(KMixer, "Mixer");


CSndMixer* CSndMixer::NewL()
  {
  CSndMixer* self = new( ELeave )CSndMixer();
  CleanupStack::PushL( self );
  self->ConstructL();
  CleanupStack::Pop( self );
  return self;
  }


void CSndMixer::ConstructL()
  {
  iShared.iMainVolume = ApplicationPrefs::volume;  // default main volume

  User::LeaveIfError( iShared.iAliveMutex.CreateLocal( 1 ) );
  User::LeaveIfError( iShared.iMutex.CreateLocal() );
  User::LeaveIfError( iMixerThread.Create( KMixer,
        CMixerThread::ThreadFunction,
        KDefaultStackSize,
        KMinHeapSize, 
        KMinHeapSize+1000000, 
        &iShared));
  
  iMixerThread.SetPriority( EPriorityMore );
  iMixerThread.Resume();
  iPaused = ETrue;
  }


CSndMixer::CSndMixer()
  {
  }


CSndMixer::~CSndMixer()
  {
  SendCmd( ECmdDestroyMixer );
  iShared.iAliveMutex.Wait();
  iShared.iAliveMutex.Close();
  iShared.iMutex.Close();
  }


void CSndMixer::Pause()
  {
  RDebug::Print( _L("CSndMixer::Pause") );

  if( iPaused )
    {
    return;
    }
  iPaused = ETrue;
  SendCmd( ECmdStopMixer );
  }


void CSndMixer::Resume()
  {
  RDebug::Print( _L("CSndMixer::Resume") );
    
  if( !iPaused )
    {
    return;
    }   
  iPaused = EFalse;
  SendCmd( ECmdStartMixer );
  }
  

void CSndMixer::Play( const TSample& aSample, TInt aChannel, TInt aFrequency, TInt aVolume )
  {
  iShared.iMutex.Wait();
  iShared.iPlayStarted[ aChannel ] = ETrue;
  iShared.iSample[ aChannel ] = aSample;
  iShared.iFrequency[ aChannel ] = aFrequency;
  iShared.iVolume[ aChannel ] = aVolume;
  iShared.iMutex.Signal();
  }


void CSndMixer::Stop( TInt aChannel )
  {
  iShared.iMutex.Wait();
  iShared.iPlayStarted[ aChannel ] = ETrue;
  iShared.iSample[ aChannel ] = iEmptySample;
  iShared.iMutex.Signal();  
  }


void CSndMixer::SetVolume( TInt aVolume )
  {
  iShared.iMutex.Wait();
  iShared.iMainVolume = aVolume;
  iShared.iMutex.Signal();  
  }


TInt CSndMixer::Volume()
  {
  return iShared.iMainVolume;
  }


void CSndMixer::SendCmd( TMixerCmd aCmd )
  {
  iShared.iMutex.Wait();
  iShared.iCmd = aCmd;
  iShared.iMutex.Signal();
  iShared.iExc = EExcUserInterrupt;
    
  TRequestStatus* status = iShared.iStatusPtr;
  if(status->Int() == KRequestPending )
    {
    iMixerThread.RequestComplete(status, KErrNone);
    }
  }

// End of File
