/*
 * Copyright  2008 Nokia Corporation.
 */

#include "SoundMixerContainer.h"
#include <eikenv.h>
#include "CSndMixer.h"
#include "CWavLoader.h"
#include "../engine/ApplicationPrefs.h"

// Profile settings
#include <centralrepository.h>
#include <ProfileEngineSDKCRKeys.h>

_LIT(KMusicFile, "bgm.wav");
_LIT(KEffectFile, "smlboom.wav");
_LIT(KEffect2File, "midboom.wav");
_LIT(KEffect3File, "lrgboom.wav");

CSoundMixerContainer::CSoundMixerContainer()
    {

    }

CSoundMixerContainer* CSoundMixerContainer::NewL()
  {
	CSoundMixerContainer* self = NewLC();
  CleanupStack::Pop( self );
  return self;
  }


CSoundMixerContainer* CSoundMixerContainer::NewLC()
  {
	CSoundMixerContainer* self = new( ELeave )CSoundMixerContainer();
  CleanupStack::PushL( self );
  self->ConstructL();
  return self;
  }

void CSoundMixerContainer::ConstructL()
    {
    // load sound samples
    CWavLoader* wavLoader = CWavLoader::NewLC();

    iMusicSample = wavLoader->LoadL(KMusicFile());
    iEffectSmlBoom = wavLoader->LoadL(KEffectFile());
    iEffectMidBoom = wavLoader->LoadL(KEffect2File());
    iEffectLrgBoom = wavLoader->LoadL(KEffect3File());
    
    CleanupStack::PopAndDestroy( wavLoader );

    // create sound mixer
    iSndMixer = CSndMixer::NewL();

    // make iMusicSample repeat itself
    iMusicSample.iRepEnd = iMusicSample.iLength;

    // and play music on channel 0 at 16KHz with volume 256 ( max )
    // if current profile is not silent 
    if (!IsProfileSilent() && ApplicationPrefs::playMusic==true ) { 
    	   iSndMixer->Play( iMusicSample, 0, 16000, 256 );
    }
  
    // make this class observe changes is foreground events
    // to stop blocks and sound when application is inactive
    //CEikonEnv::Static()->AddForegroundObserverL( *this );
    }

CSoundMixerContainer::~CSoundMixerContainer()
    {
    delete iSndMixer;
    delete iMusicSample.iData;
    delete iEffectSmlBoom.iData;
    delete iEffectMidBoom.iData;
    delete iEffectLrgBoom.iData;
    }

void CSoundMixerContainer::HandleGainingForeground()
  {
  // application gets focused
  // this function is called first time when application starts
  // that's why there's no need to start timer and sound in constructor
  // if the mixer is manually stopped, it should not be started when focus is gained
    StartMixer();
  }


void CSoundMixerContainer::HandleLosingForeground()
  {
  // application loses focus
  // so stop moving blocks and playing sound
  StopMixer();
  }

void CSoundMixerContainer::StartBGM()
  {
	iSndMixer->Play( iMusicSample, 0, 16000, 256 );
  }


void CSoundMixerContainer::StopBGM()
  {
	iSndMixer->Stop(0);
  }


void CSoundMixerContainer::StartMixer()
  {
  if (!IsProfileSilent())
    {
    iSndMixer->Resume();
    }
  }

void CSoundMixerContainer::StopMixer()
  {
  iSndMixer->Pause();
  }


void CSoundMixerContainer::PlaySmallBoom()
  {
  iSndMixer->Play( iEffectSmlBoom, 1, 16000, 256 );
  }

void CSoundMixerContainer::PlayMidBoom()
  {
  iSndMixer->Play( iEffectMidBoom, 2, 16000, 256 );
  }

void CSoundMixerContainer::PlayLargeBoom()
  {
  iSndMixer->Play( iEffectLrgBoom, 3, 16000, 256 );
  }

void CSoundMixerContainer::SetVolume(TInt volume)
  {
  iSndMixer->SetVolume( volume );
  }

void CSoundMixerContainer::VolumeUp()
  {
  TInt volume = iSndMixer->Volume();
  volume += 10;
  if( volume > 100 ) volume = 100;

  iSndMixer->SetVolume( volume );
  }

void CSoundMixerContainer::VolumeDown()
  {
  TInt volume = iSndMixer->Volume();
  volume -= 10;
  if( volume < 0 ) volume = 0;
  
  iSndMixer->SetVolume( volume );
  }


// ---------------------------------------------------------
// CSoundMixerContainer::IsProfileSilent()
// Used for determining whether current profile is silent.
// This means that sounds should not be played.
// ---------------------------------------------------------

TBool CSoundMixerContainer::IsProfileSilent()
  {
	return false;
    CRepository* centralRepository = CRepository::NewLC(KCRUidProfileEngine);
    TInt value;
    // Get the ringing type of the active profile
    User::LeaveIfError(centralRepository->Get(KProEngActiveRingingType, value));
    CleanupStack::PopAndDestroy(centralRepository);
    return (value == 4);  // 4 = Silent
  }

// End of File
